unit main;

{

Copyright (C) 1999  Matthew Collins

matthew@janes.demon.co.uk
http://www.janes.demon.co.uk/

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

}


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Registry, rmClass, Gauges;

const
  WM_DOFILE = WM_APP + 100;
  WM_DONEXT = WM_APP + 101;

type
  TfrmMain = class(TForm)
    dlgOpen: TOpenDialog;
    pnlBlock: TPanel;
    lstBlock: TListBox;
    Splitter1: TSplitter;
    pnlHeader: TPanel;
    lstDetails: TListBox;
    pnlButtons: TPanel;
    btnOpen: TButton;
    lblFileName: TLabel;
    lblHead: TLabel;
    lblBlock: TLabel;
    Gauge: TGauge;
    lblCurrent: TLabel;
    lblMax: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnOpenClick(Sender: TObject);
  private
    TheRegistry : TRegistry;
    FFileName: String;
    BlockNo : Integer;
    BlockSize : Integer;
    FileSize : Integer;
    FOutFile: String;
    procedure SetFileName(const Value: String);
    procedure WmDoFile (var Msg : TMessage) ; message WM_DOFILE ;
    procedure WmDoNext (var Msg : TMessage) ; message WM_DONEXT ;
    procedure SetOutFile(const Value: String);
    { Private declarations }
  public
    property FileName : String read FFileName write SetFileName;
    property OutFile : String read FOutFile write SetOutFile;
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.DFM}

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  TheRegistry := TRegistry.Create;
  TheRegistry.RootKey := HKEY_CURRENT_USER;
  if not TheRegistry.OpenKey('Software\Matthew\RM-Cobol Viewer',True) then
  begin
    TheRegistry.Free;
    TheRegistry := nil;
  end
  else
  begin
    if TheRegistry.ValueExists('FileName') then
      FileName := TheRegistry.ReadString('FileName');
  end;
  BlockNo := 1;
end;

procedure TfrmMain.SetFileName(const Value: String);
begin
  FFileName := Value;
  if assigned (TheRegistry) then
  begin
    TheRegistry.WriteString('FileName',Value);
  end;
  lblFileName.Caption := Value;
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  TheRegistry.Free;
  TheRegistry := nil;
end;

procedure TfrmMain.btnOpenClick(Sender: TObject);
begin
  if length (FileName) > 0 then
  begin
    dlgOpen.InitialDir := ExtractFilePath(FileName);
    dlgOpen.FileName := ExtractFileName(FileName);
  end;
  if dlgOpen.Execute then
  begin
    FileName := dlgOpen.FileName;
    OutFile := FileName + '.out';
    BlockNo := 0;
    BlockSize := 0;
    FileSize := 0;
    lstBlock.Items.Clear;
    lstDetails.Items.Clear;
    lstBlock.Items.BeginUpdate;
    PostMessage (Self.Handle,WM_DOFILE,0,0);
  end;
end;

procedure TfrmMain.WmDoFile(var Msg: TMessage);
var rmHeader : TrmHeader;
    rmData : TrmData;
    rmKey : TrmKey;
    lGotOne : Boolean;
    nBlockID : Integer;
begin
  if length (FileName) > 0 then
  begin
    rmHeader := TrmHeader.Create;
    try
      if lstDetails.Items.Count = 0 then
      begin
        rmHeader.FileName := FileName;
        FileSize := rmHeader.Stream.Size;
        Gauge.MinValue := 0;
        Gauge.MaxValue := rmHeader.Stream.Size;
        lblMax.Caption := IntToStr (Gauge.MaxValue);
        Gauge.Progress := 0;
        with lstDetails.Items do
        begin
          add ('Signature = ' + rmHeader.Signature);
          add ('Min Record = ' + inttostr(rmHeader.MinRecord));
          add ('Max Record = ' + inttostr(rmHeader.MaxRecord));
          if rmHeader.Compression then
            add ('Data Compression on');
          add ('Space code = ' + inttostr (rmHeader.SpaceCode));
          add ('Number code = ' + inttostr(rmHeader.NumberCode));
          if rmHeader.KeyCompression then
            add ('Key Compression on');
          add ('Key Number code = ' + inttostr(rmHeader.KeyNumberCode));
          BlockSize := rmHeader.BlockSize;
          add ('Block size = ' + inttostr(BlockSize));
          add ('Block increment = ' + inttostr(rmHeader.BlockIncrement));
          add ('Block contains = ' + inttostr(rmHeader.BlockContains));
          add ('Allocation Increment = ' + inttostr(rmHeader.AllocationInc));
          add ('Index Blocks = ' + inttostr(rmHeader.IndexBlock));
          add ('No records = ' + inttostr(rmHeader.NoRecords));
          add ('Integrity flag = ' + inttostr(rmHeader.IntegrityFlag));
          add ('Primary Key Offset = ' + inttostr(rmHeader.PriKeyOffSet));
          add ('Primary Key Length = ' + inttostr(rmHeader.PriKeyLength));
          add ('Empty Blocks = ' + inttostr(rmHeader.EmptyBlocks));
        end;
      end;
      PostMessage (Self.Handle,WM_DONEXT,0,0);
      lGotOne := False;
      Gauge.Progress := BlockNo * BlockSize;
      lblCurrent.Caption := IntToStr (Gauge.Progress);
      rmData := TrmData.Create;
      try
        rmData.FileName := FileName;
        rmData.Offset := BlockNo;
        rmData.BlockSize := BlockSize;
        nBlockID := rmData.BlockID;
        if rmData.ValidID then
        begin
          lGotOne := True;
          with lstBlock.Items do
          begin
            add ('Data Block Number = ' + inttostr (BlockNo));
          end;
          rmData.OutPutRecords(OutFile);
        end;
      finally
        rmData.Free;
      end;
      rmKey := TrmKey.Create;
      try
        rmKey.FileName := FileName;
        rmKey.OffSet := BlockNo;
        rmKey.BlockSize := BlockSize;
        if rmKey.ValidID then
        begin
          lGotOne := True;
          with lstBlock.Items do
          begin
            add ('Key  Block Number = ' + inttostr (BlockNo));
          end;
        end;
      finally
        rmKey.Free;
      end;
      if not lGotOne then
      begin
        lstBlock.Items.Add (IntToStr(nBlockId) + ' - ' + inttostr(BlockNo))
      end;
      lstBlock.ItemIndex := lstBlock.Items.Count - 1;
    finally
      rmHeader.Free;
    end;
  end;
end;

procedure TfrmMain.SetOutFile(const Value: String);
var TempStream : TFileStream;
begin
  FOutFile := Value;
  TempStream := TFileStream.Create ( FOutFile, fmCreate );
  TempStream.Free;
end;

procedure TfrmMain.WmDoNext(var Msg: TMessage);
begin
  Application.ProcessMessages;
  Inc (BlockNo);
  if (BlockNo * BlockSize) <= FileSize then
  begin
    PostMessage (Self.Handle,WM_DOFILE,0,0);
  end
  else
    lstBlock.Items.EndUpdate;
end;

end.
