unit rmClass;

{

Copyright (C) 1999  Matthew Collins

matthew@janes.demon.co.uk
http://www.janes.demon.co.uk/

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

}


interface

uses Classes, SysUtils;

type
  TrmBase = class
  private
    FFileName: String;
    procedure SetFileName(const Value: String);
  protected
    FileStream: TFileStream;
  public
    Constructor Create; virtual;
    Destructor Destroy; override;
    function GetByte (const Offset : Integer) : Byte;
    function GetWord (const Offset : Integer) : Word;
  published
    property FileName : String read FFileName write SetFileName;
    property Stream : TFileStream read FileStream;
  end;
  TrmHeader = class (TrmBase)
  private
    function GetSignature : ShortString;
    function GetAllocationInc: Word;
    function GetBlockContains: Byte;
    function GetBlockIncrement: Word;
    function GetBlockSize: Word;
    function GetCompression: Boolean;
    function GetEmptyBlocks: Word;
    function GetIndexBlocks: Word;
    function GetIntegrityFlag: Byte;
    function GetKeyCompression: Boolean;
    function GetKeyNumberCode: Byte;
    function GetMaxRecord: Word;
    function GetMinRecord: Word;
    function GetNoRecords: Word;
    function GetNumberCode: Byte;
    function GetPriKeyLength: Word;
    function GetPriKeyOffSet: Word;
    function GetSpaceCode: Byte;
  protected
  public
    Constructor Create; override;
    function AltKeyOffset (const Key : Byte) : Word;
    function AltKeyLength (const Key : Byte) : Word;
    property Signature : ShortString read GetSignature;
    property MinRecord : Word read GetMinRecord;
    property MaxRecord : Word read GetMaxRecord;
    property Compression : Boolean read GetCompression;
    property SpaceCode : Byte read GetSpaceCode;
    property NumberCode : Byte read GetNumberCode;
    property KeyCompression : Boolean read GetKeyCompression;
    property KeyNumberCode : Byte read GetKeyNumberCode;
    property BlockSize : Word read GetBlockSize;
    property BlockIncrement : Word read GetBlockIncrement;
    property BlockContains : Byte read GetBlockContains;
    property AllocationInc : Word read GetAllocationInc;
    property IndexBlock : Word read GetIndexBlocks;
    property NoRecords : Word read GetNoRecords;
    property IntegrityFlag : Byte read GetIntegrityFlag;
    property PriKeyOffSet : Word Read GetPriKeyOffSet;
    property PriKeyLength : Word read GetPriKeyLength;
    property EmptyBlocks : Word read GetEmptyBlocks;
  published
  end;
  TrmDataBase = class(TrmBase)
  private
    FOffset: Word;
    FBlockSize: Word;
    procedure CheckEnough;
    procedure SetOffset(const Value: Word);
    procedure SetBlockSize(const Value: Word);
    function GetBlockID: Byte;
    function GetKeyNo: Byte;
    function GetPosition: Integer;
  protected
    lGotEnough: Boolean;
    nBlockID : Integer;
    function DeCompress ( DataBlock : Pointer ; const nSize : Integer ) : String;
  public
    constructor Create ; override;
    function ValidID : Boolean; virtual;
  published
    property Position : Integer read GetPosition;
    property Offset : Word read FOffset write SetOffset;
    property BlockSize : Word read FBlockSize write SetBlockSize;
    property BlockID : Byte read GetBlockID;
    property KeyNo : Byte read GetKeyNo;
  end;
  TrmData = class(TrmDataBase)
  private
    function GetCharactersFollowing: Word;
  protected

  public
    constructor Create ; override;
    procedure OutPutRecords (const OutPutFile : String);
    function ValidID : Boolean; override;
  published
    property CharactersFollowing : Word read GetCharactersFollowing;
  end;
  TrmKey = class(TrmDataBase)
  private
    function GetCharactersFollowing: Word;
    function GetNextIndexBlock: Word;
  protected
  public
    constructor Create ; override;
  published
    property NextIndexBlock : Word read GetNextIndexBlock;
    property CharactersFollowing : Word read GetCharactersFollowing;
  end;




implementation

{ TrmBase }

constructor TrmBase.Create;
begin

end;

destructor TrmBase.Destroy;
begin
  try
    if assigned (FileStream) then
    begin
      FileStream.Free;
      FileStream := nil;
    end;
  finally
    inherited Destroy;
  end;
end;

procedure TrmBase.SetFileName(const Value: String);
begin
  FFileName := Value;
  if assigned (FileStream) then
  begin
    FileStream.Free;
    FileStream := nil;
  end;
  try
    FileStream := TFileStream.Create(FFileName,fmOpenRead or fmShareDenyNone);
  except
    on EFopenError do begin
                        FileStream := nil;
                        FFileName := '';
                        raise;
                      end;
  end;
end;

function TrmBase.GetWord(const Offset: Integer): Word;
var PWord : ^Word;
    PWordLo, PWordHi : Pointer;
begin
  Result := 0;
  if assigned (FileStream) then
  begin
    FileStream.Seek(OffSet-1,soFromBeginning);
    new (PWord);
    try
      PWordHi := PWord;
      PWordLo := PWord;
      inc (Integer(PWordHi));
      FileStream.Read(PWordHi^,1);
      FileStream.Read(PWordLo^,1);
      Result := PWord^;
    finally
      Dispose (PWord);
    end;
  end;
end;

function TrmBase.GetByte(const Offset: Integer): Byte;
var PByte : ^Byte;
begin
  Result := 0;
  if assigned (FileStream) then
  begin
    FileStream.Seek(OffSet-1,soFromBeginning);
    new (PByte);
    try
      FileStream.Read(PByte^,1);
      Result := PByte^;
    finally
      Dispose (PByte);
    end;
  end;
end;

{ TrmHeader }

function TrmHeader.AltKeyLength(const Key: Byte): Word;
begin
  Result := GetWord (257+(key*36));
end;

function TrmHeader.AltKeyOffset(const Key: Byte): Word;
begin
  Result := GetWord (259+(key*36));
end;

constructor TrmHeader.Create;
begin
  inherited Create;
end;

function TrmHeader.GetAllocationInc: Word;
begin
  Result := GetWord(257);
end;

function TrmHeader.GetBlockContains: Byte;
begin
  Result := GetByte(31);
end;

function TrmHeader.GetBlockIncrement: Word;
begin
  Result := GetWord(29);
end;

function TrmHeader.GetBlockSize: Word;
begin
  Result := GetWord(27);
end;

function TrmHeader.GetCompression: Boolean;
begin
  if GetByte (24) = 2 then
    Result := True
  else
    Result := False;
end;

function TrmHeader.GetEmptyBlocks: Word;
begin
  Result := GetWord(261);
end;

function TrmHeader.GetIndexBlocks: Word;
begin
  Result := Getword(45);
end;

function TrmHeader.GetIntegrityFlag: Byte;
begin
  Result := GetByte(55);
end;

function TrmHeader.GetKeyCompression: Boolean;
begin
  if GetByte(24) = 2 then
    Result := True
  else
    Result := False;
end;

function TrmHeader.GetKeyNumberCode: Byte;
begin
  Result := GetByte (25);
end;

function TrmHeader.GetMaxRecord: Word;
begin
  Result := GetWord (19);
end;

function TrmHeader.GetMinRecord: Word;
begin
  Result := GetWord(17);
end;

function TrmHeader.GetNoRecords: Word;
begin
  Result := GetWord(53);
end;

function TrmHeader.GetNumberCode: Byte;
begin
  Result := GetByte(23);
end;

function TrmHeader.GetPriKeyLength: Word;
begin
  Result := GetWord(259);
end;

function TrmHeader.GetPriKeyOffSet: Word;
begin
  Result := GetWord(257);
end;

function TrmHeader.GetSignature: ShortString;
var PChars : array [0..3] of char;
begin
  Result := '';
  if assigned (FileStream) then
  begin
    FileStream.Seek(7-1,soFromBeginning);
    FileStream.Read(PChars,4);
    Result := PChars;
  end;
end;

function TrmHeader.GetSpaceCode: Byte;
begin
  Result := GetByte(22);
end;


{ TrmDataBase }

procedure TrmDataBase.CheckEnough;
begin
  if (FBlockSize > 0) then
    lGotEnough := True
  else
    lGotEnough := False;
end;

constructor TrmDataBase.Create;
begin
  inherited Create;
  lGotEnough := False;
  FBlockSize := 0;
  FOffset := 0;
  nBlockID := 0;
end;

function TrmDataBase.DeCompress(DataBlock: Pointer; const nSize : integer): String;
var nCount, nMax, nLoop : Integer;
    cOutPut : String;
    pIn : PChar;
    fill_char : Char;
    lDoIt : Boolean;
    fill_num : Integer;
begin
  pIn := DataBlock;
  nCount := nSize;
  cOutPut := '';
  fill_char := #0;
  fill_num := 0;
  while nCount > 0 do
  begin
    nMax := Byte(pIn^);
    lDoIt := True;
    if nMax > 127 then
    begin
      if nMax > 231 then
      begin
        inc(pIn);
        dec (nCount);
        fill_char := pIn^;
        fill_num := nMax - 230;
      end
      else if nMax > 207 then
      begin
        fill_char := #0;
        fill_num := nMax - 210;
      end
      else if nMax > 191 then
      begin
        fill_char := '0';
        fill_num := nMax - 190;
      end
      else begin
        fill_char := ' ';
        fill_num := nMax - 126;
      end;
    end
    else
    begin
      lDoIt := False;
      for nLoop := 1 to nMax do
      begin
        inc (pIn);
        dec (nCount);
        cOutPut := cOutPut + pIn^;
      end;
    end;
    if lDoIt then
      for nLoop := 1 to fill_num do
        cOutPut := cOutPut + fill_char;
    inc (pIn);
    dec (nCount);
  end;
  Result := cOutPut;
end;

function TrmDataBase.GetBlockID: Byte;
begin
  if lGotEnough then
    Result := GetByte ( Position + 2 )
  else
    Result := 0;
end;

function TrmDataBase.GetKeyNo: Byte;
begin
  if lGotEnough then
    Result := GetByte ( Position + 1 )
  else
    Result := 0;
end;

function TrmDataBase.GetPosition: Integer;
begin
  Result := (FOffset*FBlockSize) ;
end;

procedure TrmDataBase.SetBlockSize(const Value: Word);
begin
  FBlockSize := Value;
  CheckEnough;
end;

procedure TrmDataBase.SetOffset(const Value: Word);
begin
  FOffset := Value;
  CheckEnough;
end;

function TrmDataBase.ValidID: Boolean;
begin
  if BlockID <> nBlockID then
    Result := False
  else
    Result := True;
end;

{ TrmData }

constructor TrmData.Create;
begin
  nBlockID := 6;
end;

function TrmData.GetCharactersFollowing: Word;
begin
  if lGotEnough then
    Result := GetWord ( Position + 7 )
  else
    Result := 0;
end;

function TrmData.ValidID: Boolean;
begin
  Result := Inherited ValidID;
  if not Result then
    if BlockID = 7 then Result := True;
end;

procedure TrmData.OutPutRecords(const OutPutFile: String);
var nStartPos : Integer;
    nBytesLeft : Integer;
    OutStream : TFileStream;
    nSize, nSkip : Integer;
    cData : Pointer;
    cOutString : PChar;
    CRLF : array [0..1] of char;
begin
  CRLF := #13#10;
  OutStream := nil;
  try
    try
      OutStream := TFileStream.Create(OutPutFile, fmOpenWrite + fmShareDenyNone);
    except
      on EFOpenError do Exit;
    end;
    OutStream.Seek (0,soFromEnd);
    nStartPos := 11 ;
    nBytesLeft := CharactersFollowing - 11;
    nSkip := 0;
    while GetWord (Position + nStartPos) = 0 do
    begin
      inc (nSkip,4);
      inc (nStartPos,4);
      dec (nBytesLeft,4);
    end;
    while nBytesLeft > 0 do
    begin
      nSize := GetWord (nStartPos + Position);
      GetMem (cData,nSize);
      try
        FileStream.Read(cData^,nSize);
        dec (nBytesLeft,nSize+nSkip+4);
        inc (nStartPos,nSize+nSkip+4);
        cOutString := pchar (DeCompress(cData,nSize));
        OutStream.Write(cOutString^,length(cOutString));
        OutStream.Write(CRLF,2);
      finally
        FreeMem (cData);
      end;
    end;
  finally
    OutStream.Free;
  end;
end;

{ TrmKey }

constructor TrmKey.Create;
begin
  Inherited Create;
  nBlockID := 5;
end;

function TrmKey.GetCharactersFollowing: Word;
begin
  if lGotEnough then
    Result := GetWord ( Position + 7 )
  else
    Result := 0;
end;

function TrmKey.GetNextIndexBlock: Word;
begin
  if lGotEnough then
    Result := GetWord ( Position + 5 )
  else
    Result := 0;
end;

end.
